<?php
//
// Definition of eZIdentifierType class
//
// Created on: <28-Aug-2003 11:43:09 br>
//
// Copyright (C) 1999-2003 eZ systems as. All rights reserved.
//
// This source file is part of the eZ publish (tm) Open Source Content
// Management System.
//
// This file may be distributed and/or modified under the terms of the
// "GNU General Public License" version 2 as published by the Free
// Software Foundation and appearing in the file LICENSE.GPL included in
// the packaging of this file.
//
// Licencees holding valid "eZ publish professional licences" may use this
// file in accordance with the "eZ publish professional licence" Agreement
// provided with the Software.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING
// THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE.
//
// The "eZ publish professional licence" is available at
// http://ez.no/products/licences/professional/. For pricing of this licence
// please contact us via e-mail to licence@ez.no. Further contact
// information is available at http://ez.no/home/contact/.
//
// The "GNU General Public License" (GPL) is available at
// http://www.gnu.org/copyleft/gpl.html.
//
// Contact licence@ez.no if any conditions of this licencing isn't clear to
// you.
//

/*! \file ezidentifiertype.php
*/

/*!
  \class eZIdentifierType ezidentifiertype.php
  \brief The class eZIdentifierType does

*/

include_once( "kernel/classes/ezdatatype.php" );
include_once( "lib/ezutils/classes/ezintegervalidator.php" );

define( "EZ_DATATYPESTRING_PRETEXT_FIELD", "data_text1" );
define( "EZ_DATATYPESTRING_PRETEXT_VARIABLE", "_ezidentifier_pretext_value_" );

define( "EZ_DATATYPESTRING_POSTTEXT_FIELD", "data_text2" );
define( "EZ_DATATYPESTRING_POSTTEXT_VARIABLE", "_ezidentifier_posttext_value_" );

define( "EZ_DATATYPESTRING_START_VALUE_FIELD", "data_int1" );
define( "EZ_DATATYPESTRING_START_VALUE_VARIABLE", "_ezidentifier_start_integer_value_" );

define( "EZ_DATATYPESTRING_DIGITS_FIELD", "data_int2" );
define( "EZ_DATATYPESTRING_DIGITS_VARIABLE", "_ezidentifier_digits_integer_value_" );

define( "EZ_DATATYPESTRING_IDENTIFIER_FIELD", "data_int3" );
define( "EZ_DATATYPESTRING_IDENTIFIER_VARIABLE", "_ezidentifier_identifier_value_" );

define( "EZ_DATATYPESTRING_IDENTIFIER", "ezidentifier" );

class eZIdentifierType extends eZDataType
{
    /*!
     Constructor
    */
    function eZIdentifierType()
    {
        $this->eZDataType( EZ_DATATYPESTRING_IDENTIFIER,
                           ezi18n( 'kernel/classes/datatypes', "Identifier", 'Datatype name' ),
                           array( 'serialize_supported' => false ) );

        $this->IntegerValidator = new eZIntegerValidator();
    }

    /*!
     Validates the input and returns true if the input was
     valid for this datatype.
    */
    function validateObjectAttributeHTTPInput( &$http, $base, &$contentObjectAttribute )
    {
    }

    function fetchObjectAttributeHTTPInput( &$http, $base, &$contentObjectAttribute )
    {
    }

    /*!
     Store the content. Since the content has been stored in function fetchObjectAttributeHTTPInput(),
     this function is with empty code.
    */
    function storeObjectAttribute( &$contentObjectattribute )
    {
    }

    /*!
     Returns the content.
    */
    function &objectAttributeContent( &$contentObjectAttribute )
    {
        $content = $contentObjectAttribute->attribute( "data_text" );
        if ( trim( $content ) == '' )
        {
            $contentClassAttribute =& $contentObjectAttribute->contentClassAttribute();
            $content = eZIdentifierType::generateIdentifierString( $contentClassAttribute, false );
        }
        return $content;
    }

    /*!
      Validates the input and returns true if the input was
      valid for this datatype.
    */
	function validateClassAttributeHTTPInput( &$http, $base, &$classAttribute )
	{
        $startValueName = $base . EZ_DATATYPESTRING_START_VALUE_VARIABLE . $classAttribute->attribute( "id" );
        $digitsName = $base . EZ_DATATYPESTRING_DIGITS_VARIABLE . $classAttribute->attribute( "id" );

        if ( $http->hasPostVariable( $startValueName ) and
             $http->hasPostVariable( $digitsName ) )
        {
            $startValueValue = str_replace( " ", "", $http->hasPostVariable( $startValueName ) );
            $digitsValue = str_replace( " ", "", $http->hasPostVariable( $digitsName ) );

            if ( $startValueValue == "" and $digitsValue == "" )
            {
                return EZ_INPUT_VALIDATOR_STATE_ACCEPTED;
            }
            else
            {
                $startValueValueState = $this->IntegerValidator->validate( $startValueValue );
                $digitsValueState = $this->IntegerValidator->validate( $digitsValue );

                if ( ( $startValueValueState == EZ_INPUT_VALIDATOR_STATE_ACCEPTED ) and
                     ( $startValueValue >= 0 ) and
                     ( $digitsValueState == EZ_INPUT_VALIDATOR_STATE_ACCEPTED ) and
                     ( $digitsValue >= 0 ) )
                {
                    return EZ_INPUT_VALIDATOR_STATE_ACCEPTED;
                }
            }
        }

        return EZ_INPUT_VALIDATOR_STATE_INVALID;
    }


	/*!
	 \reimp
	*/
	function fetchClassAttributeHTTPInput( &$http, $base, &$classAttribute )
	{
        $startValueName = $base . EZ_DATATYPESTRING_START_VALUE_VARIABLE . $classAttribute->attribute( "id" );
        $digitsName = $base . EZ_DATATYPESTRING_DIGITS_VARIABLE . $classAttribute->attribute( "id" );
		$preTextName = $base . EZ_DATATYPESTRING_PRETEXT_VARIABLE . $classAttribute->attribute( "id" );
		$postTextName = $base . EZ_DATATYPESTRING_POSTTEXT_VARIABLE . $classAttribute->attribute( "id" );

        if ( $http->hasPostVariable( $startValueName ) and
             $http->hasPostVariable( $digitsName ) and
             $http->hasPostVariable( $preTextName ) and
             $http->hasPostVariable( $postTextName ) )
        {
            $startValueValue = str_replace( " ", "", $http->postVariable( $startValueName ) );
            $digitsValue = str_replace( " ", "", $http->postVariable( $digitsName ) );
            $preTextValue =  $http->postVariable( $preTextName );
            $postTextValue = $http->postVariable( $postTextName );

            $classAttribute->setAttribute( EZ_DATATYPESTRING_START_VALUE_FIELD, $startValueValue );
            $classAttribute->setAttribute( EZ_DATATYPESTRING_DIGITS_FIELD, $digitsValue );
            $classAttribute->setAttribute( EZ_DATATYPESTRING_PRETEXT_FIELD, $preTextValue );
            $classAttribute->setAttribute( EZ_DATATYPESTRING_POSTTEXT_FIELD, $postTextValue );

            $originalClassAttribute = eZContentClassAttribute::fetch( $classAttribute->attribute( 'id' ), true, 0 );
            if ( ( $classAttribute->attribute( 'data_int4' ) == 0 ) and
                 ( get_class( $originalClassAttribute ) == 'ezcontentclassattribute' ) )
            {
                $classAttribute->setAttribute( EZ_DATATYPESTRING_IDENTIFIER_FIELD,
                                               $originalClassAttribute->attribute( EZ_DATATYPESTRING_IDENTIFIER_FIELD ) );
            }
            else
            {
                $classAttribute->setAttribute( EZ_DATATYPESTRING_IDENTIFIER_FIELD,
                                               $classAttribute->attribute( EZ_DATATYPESTRING_START_VALUE_FIELD ) );
            }

        }
        return true;
    }

    /*!
     Returns the meta data used for storing search indices.
    */
    function metaData( $contentObjectAttribute )
    {
        return $contentObjectAttribute->attribute( "data_text" );
    }

    /*!
     Returns the text.
    */
    function title( &$contentObjectAttribute )
    {
        return  $contentObjectAttribute->attribute( "data_text" );
    }

    /*!
     \reimp
    */
    function isIndexable()
    {
        return true;
    }


    /*!
      The value should be created and incremented when the object is published.
      No id will be generated for draft. ID should also only be generated for
      the first published version.
    */
    function onPublish( &$contentObjectAttribute, &$contentObject, &$publishedNodes )
    {
        $contentClassAttribute = $contentObjectAttribute->attribute( 'contentclass_attribute' );

        // fetch the root node
        $ret = eZIdentifierType::incrementValue( $contentClassAttribute, $contentObjectAttribute );

        $contentObjectAttribute->store();
        return $ret;
    }

    /*!
      \private
      Increment the identifiervalue for the current attribute.
    */
    function incrementValue( &$contentClassAttribute, &$contentObjectAttribute )
    {

        $retValue = true;
        $ret = array();
        $version = $contentObjectAttribute->attribute( 'version' );
        $contentClassID = $contentClassAttribute->attribute( 'id' );
        if ( $version == 1 )
        {
            $db =& eZDB::instance();
            $db->begin();

            // Ensure that we don't get another identifier with the same id.
            $db->lock( array( array( "table" => "ezcontentobject_attribute" ),
                              array( "table" => "ezcontentclass_attribute" ) ) );

            $selectQuery = "SELECT data_int3, data_int4 FROM ezcontentclass_attribute WHERE " .
                 "id='$contentClassID' AND version='0'";
            $result = $db->arrayQuery( $selectQuery );
            $identifierValue = $result[0]['data_int3'];

            // should only increment when we don't have the first version
            if ( $result[0]['data_int4'] == 0 )
            {
                $updateQuery = "UPDATE ezcontentclass_attribute SET data_int3=data_int3 + 1 WHERE " .
                     "id='$contentClassID' AND version='0'";
                $identifierValue++;
            }
            else
            {
                $updateQuery = "UPDATE ezcontentclass_attribute SET data_int4='0' WHERE " .
                     "id='$contentClassID' AND version='0'";
            }

            $ret[] = $db->query( $updateQuery );
            $ret[] = eZIdentifierType::storeIdentifierValue( $contentClassAttribute, $contentObjectAttribute, $identifierValue );

            if ( !in_array( false, $ret ) )
                $db->commit();
            else
                $db->rollback();

            $db->unlock();
        }
        else
        {
            $ret[] = eZIdentifierType::copyFromVersionOne( $contentObjectAttribute );
        }

        if ( !in_array( false, $ret ) )
            $retValue = true;

        return $retValue;
    }

    /*!
      \private
      Copy version one to the current version
    */
    function copyFromVersionOne( &$contentObjectAttribute )
    {
        $contentObjectAttributeVersionOne = eZContentObjectAttribute::fetch( $contentObjectAttribute->attribute( 'id' ), 1 );
        $contentObjectAttribute->setAttribute( 'data_text', $contentObjectAttributeVersionOne->attribute( 'data_text' ) );
        $contentObjectAttribute->setAttribute( 'data_int', $contentObjectAttributeVersionOne->attribute( 'data_int' ) );

        return true;
    }

    /*!
      \private
      Store the new value to the attribute.
    */
    function storeIdentifierValue( &$contentClassAttribute, &$contentObjectAttribute, $identifierValue )
    {
        $value = eZIdentifierType::generateIdentifierString( $contentClassAttribute, $identifierValue );
        $contentObjectAttribute->setAttribute( 'data_text', $value );
        $contentObjectAttribute->setAttribute( 'data_int', $identifierValue );
        return true;
    }

    function generateIdentifierString( &$contentClassAttribute, $identifierValue = false )
    {
        $preText = $contentClassAttribute->attribute( EZ_DATATYPESTRING_PRETEXT_FIELD );
        $postText = $contentClassAttribute->attribute( EZ_DATATYPESTRING_POSTTEXT_FIELD );
        $digits = $contentClassAttribute->attribute( EZ_DATATYPESTRING_DIGITS_FIELD );

        if ( $identifierValue )
            $midText = str_pad( $identifierValue, $digits, '0', STR_PAD_LEFT );
        else
            $midText = str_repeat( 'x', $digits );

        $value = $preText . $midText . $postText;
        return $value;
    }

    /*!
      Update the startvalue for the identifier!
     */
    function customClassAttributeHTTPAction( &$http, $action, &$contentClassAttribute )
    {
        switch ( $action )
        {
            case 'update_start_value':
            {
                $startValue = $http->postVariable( "ContentClass" . EZ_DATATYPESTRING_START_VALUE_VARIABLE .
                                                   $contentClassAttribute->attribute( 'id' ) );

                $contentClassAttribute->setAttribute( EZ_DATATYPESTRING_IDENTIFIER_FIELD, $startValue );
                $contentClassAttribute->setAttribute( 'data_int4', '1' );
            }break;
        }
    }

    function preStoreClassAttribute( &$classAttribute, $version )
    {
    }

    function preStoreDefinedClassAttribute( &$classAttribute )
    {
//        $classAttribute->setAttribute( 'data_int4', '0' );
    }

    var $IntegerValidator;
}

eZDataType::register( EZ_DATATYPESTRING_IDENTIFIER, "ezidentifiertype" );

?>
